/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * KinectInputHandler
 * For documentation, see KinectInputHandler.hpp
 */

#include <cstring>
#include <iostream>
#include <cstdio>
#include "KinectInputHandler.hpp"

KinectInputHandler::KinectInputHandler(unsigned short listenPort,
                                       LimbTracker* track) :
		osc::OscPacketListener(), lt(track),
		sock(IpEndpointName(IpEndpointName::ANY_ADDRESS, listenPort),
		     this) {
	omp_init_lock(&lock);
}

KinectInputHandler::~KinectInputHandler(){
	omp_destroy_lock(&lock);
}

void KinectInputHandler::ProcessMessage(
		const osc::ReceivedMessage& m,
		const IpEndpointName& remoteEndpoint){
	try{
		const char* addr = m.AddressPattern();
		if(!strcmp(addr, "/joint"))
			jointEvent(m);
		else if(!strcmp(addr, "/lost_user"))
			loseUserEvent(m);
	}catch(osc::Exception& e){
		std::cerr << "Error while parsing message: " <<
		             m.AddressPattern() << ": " << e.what() << "\n";
	}
}

static LimbType getLimbType(const char* joint){
	if(strcmp(joint, "l_hand") == 0) return LEFT_HAND;
	else if(strcmp(joint, "r_hand") == 0) return RIGHT_HAND;
	else return NUM_LIMBS;
}

void KinectInputHandler::jointEvent(const osc::ReceivedMessage& m){
	osc::int32 userID;
	const char* limb;
	float x, y, z;
	osc::ReceivedMessageArgumentStream args = m.ArgumentStream();

	args >> limb;
	LimbType limbID = getLimbType(limb);
	if(limbID != NUM_LIMBS){
		args >> userID >> x >> y >> z >> osc::EndMessage;
		TripleD pos(x, y, z);
		omp_set_lock(&lock);
		lt->updateLimbPos(userID, limbID, pos);
		omp_unset_lock(&lock);
	}
}

void KinectInputHandler::loseUserEvent(const osc::ReceivedMessage& m){
	osc::int32 userID;
	m.ArgumentStream() >> userID >> osc::EndMessage;
	omp_set_lock(&lock);
	lt->removeLimbSet(userID);
	omp_unset_lock(&lock);
}

