/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * LimbTracker
 * For documentation, see LimbTracker.hpp.
 */

#include <cstddef>
#include "LimbTracker.hpp"
#include "MotionSmoother.hpp"

using std::pair;

//Holds position and velocity data for a specific limb.
//All position data is smoothed.
class LimbTracker::Limb{
public:
	Limb() : ms(10) {}
	void updatePos(TripleD pos){ ms.pushFrame(pos); }

	TripleD getCurrPos() const { return ms.getPos(); }
	TripleD getCurrVel() const { return ms.getVel(); }

private:
	MotionSmoother ms;
};

//Tracks a set of limbs associated with a single user ID
class LimbTracker::LimbSet{
public:
	LimbSet(unsigned userID) : id(userID){
		for(int i = 0; i < (int)NUM_LIMBS; i++)
			limbs[i] = NULL;
	}

	~LimbSet(){
		for(int i = 0; i < (int)NUM_LIMBS; i++){
			if(NULL != limbs[i]){
				delete limbs[i];
				limbs[i] = NULL;
			}
		}
	}

	//Creates iff limb does not yet exist
	void updateLimb(LimbType limb, TripleD pos){
		if(limbs[limb] == NULL)
			limbs[limb] = new LimbTracker::Limb();
		limbs[limb]->updatePos(pos);
	}

	LimbTracker::Limb* getLimb(LimbType limb){
		if(limb != NUM_LIMBS)
			return limbs[limb];
		else
			throw IllegalLimbType();
	}

private:
	LimbTracker::Limb* limbs[NUM_LIMBS];
	unsigned id;
};

bool LimbTracker::limbSetExists(unsigned id) const {
	return sets.find(id) != sets.end();
}

bool LimbTracker::limbDataExists(unsigned id, LimbType limb) const {
	try{
		return limbSetExists(id) &&
		       sets.find(id)->second->getLimb(limb) != NULL;
	}catch(IllegalLimbType ilt){
		return false;
	}
}

bool LimbTracker::getLimbPos(unsigned id, LimbType limb,
                             TripleD& posOut) const {
	if(limbDataExists(id, limb)){
		posOut = sets.find(id)->second->getLimb(limb)->getCurrPos();
		return true;
	}else return false;
}

bool LimbTracker::getLimbVel(unsigned id, LimbType limb,
                             TripleD& velOut) const {
	if(limbDataExists(id, limb)){
		velOut = sets.find(id)->second->getLimb(limb)->getCurrVel();
		return true;
	}else return false;
}

void LimbTracker::updateLimbPos(unsigned id, LimbType limb,
                                TripleD pos){
	LimbSet* lset;
	if(limbSetExists(id))
		lset = sets.find(id)->second;
	else{
		lset = new LimbSet(id);
		sets.insert(pair<unsigned, LimbSet*>(id, lset));
	}
	lset->updateLimb(limb, pos);
	setIDs.insert(id);
}

void LimbTracker::removeLimbSet(unsigned id){
	if(limbSetExists(id)){
		sets.erase(id);
		setIDs.erase(id);
	}
}
