/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * MotionSmoother
 * For documentation, see MotionSmoother.hpp
 */

#include "MotionSmoother.hpp"

MotionSmoother::MotionSmoother(unsigned histFrames) :
		MAX_FRAMES(histFrames) {
	if(histFrames == 0) throw ZeroException();
	hist = new TripleD*[histFrames];
	for(unsigned i = 0; i < histFrames; i++)
		hist[i] = new TripleD();
	clear();
}

MotionSmoother::~MotionSmoother(){
	for(unsigned i = 0; i < MAX_FRAMES; i++)
		delete hist[i];
	delete[] hist;
}

void MotionSmoother::clear(){
	currFrames = 0;
	(*hist[0]) = vel = TripleD();
}

//Keep a running average of the frames
void MotionSmoother::pushFrame(TripleD newPos){
	currFrames = currFrames + (currFrames + 1 > MAX_FRAMES ? 0 : 1);
	newPos = newPos.scale((1 << (currFrames - 1)) + 1);
	for(int i = currFrames - 2; i >= 0; i--){
		newPos += hist[i]->scale(1 << (currFrames - i - 2));
		if(i != MAX_FRAMES - 1)
			*hist[i+1] = *hist[i];
	}
	newPos = newPos.scale(1.0/(1 << currFrames));

	vel = newPos - *hist[0];
	*hist[0] = newPos;
}

TripleD MotionSmoother::getPos() const {
	return *hist[0];
}

TripleD MotionSmoother::getVel() const {
	return vel;
}
