/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * WiiManager interfaces with the 'wiiuse' library to find, connect to and
 * retrieve input from Wii Remotes.
 *
 * wiiuse library: http://sourceforge.net/projects/wiiuse/
 */

#ifndef __WII_MANAGER_HPP__
#define __WII_MANAGER_HPP__

//For LimbType
#include "LimbTracker.hpp"
#include "wiiuse.h"

#define MAX_REMOTES 4

class WiiManager{
public:
	//Create a new WiiManager, prepared to connect to WiiMotes.
	WiiManager();

	~WiiManager();

	//Returns the number of connected WiiMotes
	unsigned getNumWiiMotes() const;

	//Returns the user ID associated with the WiiMote with the given ID
	unsigned getLimbUser(unsigned mote) const;

	//Returns the limb associated with the WiiMote with the given ID
	LimbType getLimb(unsigned mote) const;

	//The following methods returns the condition of a WiiMote with a given ID:
	//- Whether the WiiMote has been attached to a limb
	bool isLimbAttached(unsigned mote) const;
	//- Whether data from the WiiMote continues to be valid
	bool isWiiMoteValid(unsigned mote) const;
	//- Whether an event occurred since the last update
	bool eventOccurred(unsigned mote) const;
	//- Whether the given button is being held down
	bool isButtonHeld(unsigned mote, unsigned button) const;
	//- Whether the given button was just pressed during the last update
	//  (Does not seem to work as intended currently)
	bool isButtonPressed(unsigned mote, unsigned button) const;
	//- Whether the given button is released during the last update
	bool isButtonReleased(unsigned mote, unsigned button) const;

	//Searches for and connects with up to MAX_REMOTES WiiMotes
	void connectWiiMotes();

	//Updates the state of all remotes
	void update();

	//Attaches the WiiMote with the given ID to the given user ID and limb
	void attachLimb(unsigned mote, unsigned id, LimbType limb);

	//Removes WiiMote attachment to any limb, if any attachment exists.
	void detachLimb(unsigned mote);

private:
	class LimbAttachment;

	unsigned numRemotes;
	wiimote_t** motes;
	bool moteValid[MAX_REMOTES];
	LimbAttachment* limbs[MAX_REMOTES];
};

class IllegalWiiMoteIndex {};

#endif
